-- Setup
local loader = require("AdvTiledLoader/Loader")
loader.path = "maps/"
local map = loader.load("OD.tmx")
local layer = map.layers.Ground
local backgroundlayer = map.layers.Background
local Cursor = require("Cursor")
local interactlayer = map.layers.Interaction
local Player = require("Player")

local Creature = require("creature")
--Ticker for game logic
local gameticker = 0
local tickinterval = 0.1

--Store for tile types
local tiles = {}

--Store for UI elements
local uistore = {}

--store for cursor action
local currentaction = function()

end

--Player's game-data stores

local playerstore = {}

---------------------------------------------------------------------------------------------------
-- Our example class
local IsoExample = {}

---------------------------------------------------------------------------------------------------
-- Called from love.keypressed()
function IsoExample.keypressed(k)
    if k == 'w' then Cursor.move(0,-1,backgroundlayer) end
    if k == 'a' then Cursor.move(-1,0,backgroundlayer) end
    if k == 's' then Cursor.move(0,1,backgroundlayer) end
    if k == 'd' then Cursor.move(1,0,backgroundlayer) end
    if k == 'r' then IsoExample.rebuild() end
    if k == " " then currentaction() end
    
end

--changing base representation to drawable representation

--for finding neighbouring tiles 
--tilenameWNSE
--dirtwall1000 means tile neighbouring to the top

--TODO implement N NE E SE S SW W NW encoding for prettier walls

function IsoExample.rebuild()
	
	--gets different tile types
for id, tile in pairs(map.tiles) do
	if tile.properties.name then
		tiles[tile.properties.name] = tile
	end
end

	--builds walls
for x, y, tile in backgroundlayer:iterate() do	
	if tile.properties.name then
		if string.find(tile.properties.name,"wall") then
			local code = ""
			for nx = x-1, x+1 do
				for ny = y-1, y+1 do
					if (nx-x)+(ny-y) == -1 or (nx-x)+(ny-y) == 1 then --lazy to not check the ne nw se sw co-ords
						if backgroundlayer(nx,ny).properties.name and string.find(backgroundlayer(nx,ny).properties.name,"wall") then
							code = code .. "1"
						else
							code = code .. "0"
						end
					end
					
				end
			end
			if code == "0000" then code = "0" end
			layer:set(x, y, tiles[tile.properties.name..code])
		end
	end
end

end

---------------------------------------------------------------------------------------------------
-- Resets the example
function IsoExample.reset()
--    global.tx = -840
	global.tx = 840 -	map.width*map.tileWidth/2	
    global.ty = 280
    displayTime = 0
    Cursor:reset()
    
    --Initialize Players
    playerstore[1] = deepcopy(Player)
    playerstore[1].name = "Human Player"
	
    playerstore[2] = deepcopy(Player)
    playerstore[2].name = "AI Player"
	
    
    --Initialize UI
    uistore[1] = deepcopy(global.ui.panel)
    uistore[1].x = 800
    uistore[1].y = 10
    placedirtbutton = deepcopy(global.ui.button)
    placedirtbutton.label = "Place Dirt" 
    placedirtbutton.onClick = function(self) 
		currentaction = function()
		backgroundlayer:set(Cursor.tileX,Cursor.tileY,tiles["dirtwall"])
		IsoExample.rebuild()
		end
    end
    uistore[1]:add(placedirtbutton)
    
    placegoldbutton = deepcopy(global.ui.button)
    placegoldbutton.label = "Place Gold" 
    placegoldbutton.onClick = function(self) 
		currentaction = function()
		backgroundlayer:set(Cursor.tileX,Cursor.tileY,tiles["goldwall"])
		IsoExample.rebuild()
		end
    end
    uistore[1]:add(placegoldbutton)

	spawnimpbutton = deepcopy(global.ui.button)
    spawnimpbutton.label = "Spawn Imp" 
    spawnimpbutton.onClick = function(self) 
		currentaction = function()
			tempimp = deepcopy(Creature.imp)
			isocords = map:toIso( Cursor.tileX * 128 - global.tx, Cursor.tileY* 128 - global.ty) --Cursor.tileY* 128 - global.ty
			print(isocords)
			tempimp.x = Cursor.tileX*128 -global.tx - isocords
			tempimp.y = Cursor.tileY*128 -global.ty - isocords
			table.insert(playerstore[1].creaturelist,tempimp)
		end
    end
    uistore[1]:add(spawnimpbutton)
	
	IsoExample.rebuild()
end

---------------------------------------------------------------------------------------------------
-- mouseclick calllback
function IsoExample.mousepressed(x,y,mb)
    	for y,v in pairs(uistore) do
			if v.contents and #v.contents > 0 then
				for x,z in pairs(v.contents) do
					if fuzzyselect({x=love.mouse.getX(),y=love.mouse.getY()},{x=z.x,y=z.y},z.width,z.height) then 
						z.clicked= true
						z.onClick(z) 
					else
						z.clicked = false
					end
				end
			else 
				if fuzzyselect({x=love.mouse.getX(),y=love.mouse.getY()},{x=v.x,y=v.y},v.width,v.height) and v.onClick then 
					v.clicked= true
					v.onClick(v) 
				else
					v.clicked = false
				end
			end
	end
end

---------------------------------------------------------------------------------------------------
-- Update the display time for the character control instructions
function IsoExample.update(dt)
    displayTime = displayTime + dt
	
	gameticker = gameticker + dt
	
	if(gameticker>tickinterval) then 
	
		--ui hover implementation
		for y,v in pairs(uistore) do
			if v.contents and #v.contents > 0 then
				for x,z in pairs(v.contents) do
					if fuzzyselect({x=love.mouse.getX(),y=love.mouse.getY()},{x=z.x,y=z.y},z.width,z.height) then 
						z.hover = true
					else
						z.hover = false
					end
				end
			
			else
			
				if fuzzyselect({x=love.mouse.getX(),y=love.mouse.getY()},{x=v.x,y=v.y},v.width,v.height) then 
					v.hover= true 
				else
					v.hover=false
				end
			end
		end
	gameticker = 0 end
end

---------------------------------------------------------------------------------------------------
-- Add the Cursor to be drawn after the tile he's on
interactlayer:setAfterTileFunction( function(layer, tileX, tileY, drawX, drawY)
    if tileX == Cursor.tileX and tileY == Cursor.tileY then Cursor.draw(drawX, drawY) end
end )

---------------------------------------------------------------------------------------------------
-- Called from love.draw()
function IsoExample.draw()

    -- Set sprite batches
    map.useSpriteBatch = global.useBatch

    -- Scale and translate the game screen for map drawing
    local ftx, fty = math.floor(global.tx), math.floor(global.ty)
    love.graphics.push()
    love.graphics.scale(global.scale)
    love.graphics.translate(ftx, fty)
    
    -- Limit the draw range 
    if global.limitDrawing then 
        map:autoDrawRange(ftx, fty, global.scale, -50) 
    else 
        map:autoDrawRange(ftx, fty, global.scale, 50) 
    end
    
    -- Queue our Cursor to be drawn after the tile he's on and then draw the map.
    --local maxDraw = global.benchmark and 20 or 1
    --for i=1,maxDraw do 
        map:draw() 
    --end
	for i,v in pairs(playerstore) do
		for j,k in pairs(v.creaturelist) do
			k:draw()
		end
	end
    -- Reset the scale and translation.
    love.graphics.pop()
    
    for i,v in ipairs(uistore) do
		v:draw()
    end
end

---------------------------------------------------------------------------------------------------
return IsoExample
